﻿define("epi/shell/command/builder/ExpandoMenuBuilder", [
    "dojo/_base/declare",
    "dojo/_base/lang",

    "dijit/Menu",
    "dijit/form/Button",
    "dijit/form/DropDownButton",


    "../_CommandModelBindingMixin",
    "./_Builder",
    "./OptionsBuilderMixin",

    "epi/shell/widget/ExpandoMenuButton"
],

function (
    declare,
    lang,

    Menu,
    Button,
    DropDownButton,

    _CommandModelBindingMixin,
    _Builder,
    OptionsBuilderMixin,

    ExpandoMenuButton
) {

    return declare([_Builder], {
        // summary:
        //      Used to assemble an expando menu where all toggle commands is represented by expando buttons
        //      and the subsequent commands will be added as sub commands to the previous expando button
        // tags: internal

        _expandoClass: declare([ExpandoMenuButton, _CommandModelBindingMixin]),

        _dropDownClass: declare([DropDownButton, _CommandModelBindingMixin]),

        _buttonClass: declare([Button, _CommandModelBindingMixin]),

        _optionsMenuClass: declare([Menu, OptionsBuilderMixin]),

        _expandoMenu: null,

        _create: function (/*Object*/command) {
            // tags:
            //      protected, override


            //If the command specifies a widget, just return it (not that beautiful)
            if(command.widget) {
                return command.widget;
            }

            var isToggle = typeof command.active === "boolean";
            var hasOptions = command.get("options") instanceof Array;

            var options = lang.mixin({ model: command, showLabel: (command.showLabel ? command.showLabel : false) }, this.settings);

            if (hasOptions) {
                var dropDown = new this._optionsMenuClass({"class": "epi-DropDownMenu", optionItemClass: "epi-radioMenuItem", model: command});

                return new this._dropDownClass(lang.mixin({ "class": "epi-chromeless epi-mediumButton", dropDown: dropDown}, options));
            }

            if (isToggle) {
                options = lang.mixin({innerText: command.innerText}, options);
                return (this._expandoMenu = new this._expandoClass(options));
            }

            return new this._buttonClass(options);
        },

        _addToContainer: function (/*Object*/widget, /*Object*/container) {
            // summary:
            //      If the widget
            // tags:
            //      protected, override

            // Any command other than expando menu buttons will be added as children to previously created expando menu button
            if (!(widget instanceof ExpandoMenuButton) && this._expandoMenu) {
                container = this._expandoMenu;
            }

            this.inherited(arguments, [widget, container]);
        }
    });
});